# Contributing to StupidOS

First, thanks for taking the time to contribute! 😻👍

[TOC]

## Code of Conduct

This project and everyone participating in it is governed by the [StupidOS Code of Conduct](CODE_OF_CONDUCT.md). By participating, you are expected to uphold this code.
Please report unacceptable behavior to [conduct@stupidos.d0p1.eu](mailto:conduct@StupidOS.d0p1.eu).

## How Can I Contribute?

### Reporting Bugs

This section guide you through submitting a bug report for StupidOS. Following these guidelines helps maintainers and the community understand your report, reproduce the behavior, and find related reports.

Before creating bug reports, please check [this list](#before-submitting-a-bug-report) as you might find out that you don't need to create one. When you are creating a bug report, please [include as many details as possible](#how-do-i-submit-a-good-bug-report).

> **Note:** If you find a **Closed** issue that seems like it is the same thing that you're experiencing, open a new issue and include a link to the original issue in the body of your new one.

#### Before Submitting A Bug Report

* **Perform a [search](https://git.cute.engineering/d0p1/StupidOS/issues?state=open)** to see if the problem has already been reported. If it has **and the issue is still open**, add a comment to the existing issue instead of opening a new one.

#### How Do I Submit A (GOOD) Bug Report?

Bugs are tracked as [issues](https://git.cute.engineering/d0p1/StupidOS/issues).

Explain the problem and include additional details to help maintainers reproduce the problem:

* **Use a clear and descriptive title** for the issue to identify the problem.
* **Describe the exact steps which reproduce the problem** in as many details as possible.
* **Describe the behavior you observed after following the steps** and point out what exactly is the problem with that behavior.
* **Explain which behavior you expected to see instead and why.**
* **Include screenshots** which clearly demonstrate the problem if needed. 
* **include a stack trace**
* **If the problem wasn't triggered by a specific action**, describe what you were doing before the problem happened and share more information using the guidelines below.

Provide more context by answering these questions:

* **Can you reliably reproduce the issue?** If not, provide details about how often the problem happens and under which conditions it normally happens.

Include details about your configuration and environment:

* **Which version of StupidOS are you using?**
* **Are you running StupidOS in a virtual machine?** If so, which VM software are you using ?
* **Are you running StupidOS on real hardware?** If so, provide information about it.

### Your First Code Contribution

Unsure where to begin contributing to StupidOS? You can start by looking through these `good first issue` and `help wanted` issues:

* [Good first issues](https://git.cute.engineering/d0p1/StupidOS/issues?q=&type=all&sort=&labels=47&state=open&sort=mostcomment) - issues which should only require a few lines of code, and a test or two.
* [Help wanted issues](https://git.cute.engineering/d0p1/StupidOS/issues?q=&type=all&sort=&labels=11&state=open&sort=mostcomment) - issues which should be a bit more involved than `good first issue` issues.

Both issue lists are sorted by total number of comments. While not perfect, number of comments is a reasonable proxy for impact a given change will have.

## Styleguides

### Git Commit Message

Git commit message **MUST** follow [Coventional Commits convention](https://www.conventionalcommits.org/en/v1.0.0/)

Other recommandations:
- Use the present tense ("add" instead of "added")
- Use the imperative mood ("move cursor to..." not "moves cursor to...")

### Coding Style

You **MUST** follow our [coding style](https://stupidos.d0p1.eu/files/docs/coding-style-txt.html)

---

_This text is based on the [Atom's CONTRIBUTING.md](https://github.com/atom/atom/blob/master/CONTRIBUTING.md)_
